from cqase.models import ResponseModel


class SharedStepsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_SHARED_STEPS = "/shared_step/{}"
    _POST_CREATE_SHARED_STEP = "/shared_step/{}"
    _GET_SHARED_STEP_BY_CODE = "/shared_step/{}/{}"
    _DELETE_SHARED_STEP = "/shared_step/{}/{}"
    _UPDATE_SHARED_STEP = "/shared_step/{}/{}"

    def get_all(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-shared-steps
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_SHARED_STEPS.format(code)}",
        )

    def create(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-shared-step
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_SHARED_STEP.format(code)}",
            json=body,
        )

    def delete(self, code: str, hash_shared_step: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-shared-step
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}"
            f"{self._DELETE_SHARED_STEP.format(code, hash_shared_step)}",
        )

    def get_specific_shared_step(
        self, code: str, hash_shared_step: str
    ) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-shared-step
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}"
            f"{self._GET_SHARED_STEP_BY_CODE.format(code, hash_shared_step)}",
        )

    def update(self, code: str, hash_shared_step: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-shared-step
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}"
            f"{self._UPDATE_SHARED_STEP.format(code, hash_shared_step)}",
            json=body,
        )



from cqase.models import ResponseModel


class MilestoneApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_MILESTONES = "/milestone/{}"
    _POST_CREATE_MILESTONES = "/milestone/{}"
    _GET_SPECIFIC_MILESTONES = "/milestone/{}/{}"
    _DELETE_MILESTONES = "/milestone/{}/{}"
    _UPDATE_MILESTONES = "/milestone/{}/{}"

    def get_all(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-milestones
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_MILESTONES.format(code)}",
        )

    def create(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-milestone
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_MILESTONES.format(code)}",
            json=body,
        )

    def get_specific_milestone(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-milestone
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}"
            f"{self._GET_SPECIFIC_MILESTONES.format(code, uuid)}",
        )

    def delete(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-milestone
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_MILESTONES.format(code, uuid)}",
        )

    def update(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-milestone
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_MILESTONES.format(code, uuid)}",
            json=body,
        )



from cqase.models import ResponseModel


class SearchApi:
    def __init__(self, app):
        self.app = app

    _GET_SEARCH = "/v1/search"

    def search(self, params: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/search-1
        """
        return self.app.client.request(
            method="GET", url=f"{self.app.base_path}{self._GET_SEARCH}", params=params
        )



from cqase.models import ResponseModel


class CustomFieldsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_FIELDS = "/custom_field"
    _POST_NEW_FIELDS = "/custom_field"
    _GET_CUSTOM_FIELDS_BY_ID = "/custom_field/{}"
    _DELETE_CUSTOM_FIELDS = "/custom_field/{}"
    _UPDATE_CUSTOM_FIELDS = "/custom_field/{}"

    def get_all(self, params: dict = None) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-custom-fields
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_FIELDS}",
            params=params,
        )

    def create(self, body: dict):
        """
        https://developers.qase.io/reference/create-custom-field
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_NEW_FIELDS}",
            json=body,
        )

    def get_by_id(self, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-custom-field
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_CUSTOM_FIELDS_BY_ID.format(uuid)}",
        )

    def delete(self, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-custom-field
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_CUSTOM_FIELDS.format(uuid)}",
        )

    def update(self, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-custom-field
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_CUSTOM_FIELDS.format(uuid)}",
            json=body,
        )



from cqase.models import ResponseModel


class SuitesApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_SUITES = "/suite/{}"
    _POST_CREATE_SUIT = "/suite/{}"
    _GET_SUIT_BY_ID = "/suite/{}/{}"
    _DELETE_SUIT = "/suite/{}/{}"
    _UPDATE_SUIT = "/suite/{}/{}"

    def get_all(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-suites
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_SUITES.format(code)}",
        )

    def create(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-suite
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_SUIT.format(code)}",
            json=body,
        )

    def delete(self, code: str, uuid: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-suite
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}" f"{self._DELETE_SUIT.format(code, uuid)}",
        )

    def get_specific_suit(self, code: str, uuid: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-suite
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}" f"{self._GET_SUIT_BY_ID.format(code, uuid)}",
        )

    def update(self, code: str, uuid: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-suite
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}" f"{self._UPDATE_SUIT.format(code, uuid)}",
            json=body,
        )



import requests

from cqase.models import ResponseModel


class Response:
    @staticmethod
    def structure_response(response) -> ResponseModel:
        return ResponseModel(response.status_code, response.json())


class Client:
    def __init__(self, headers: dict):
        self.headers = headers

    s = requests.session()

    def request(self, method: str, url: str, **kwargs) -> ResponseModel:
        """
        Request method
        method: method for the new Request object: GET, OPTIONS, HEAD, POST, PUT, PATCH, or DELETE. # noqa
        url – URL for the new Request object.
        **kwargs:
            params – (optional) Dictionary, list of tuples or bytes to send in the query string for the Request. # noqa
            json – (optional) A JSON serializable Python object to send in the body of the Request. # noqa
            headers – (optional) Dictionary of HTTP Headers to send with the Request.
        """
        return Response.structure_response(
            self.s.request(method, url, headers=self.headers, **kwargs)
        )



import json


class BaseClass:
    def to_dict(self) -> dict:
        """
        Convert nested object to dict
        :return: dict
        """
        return json.loads(json.dumps(self, default=lambda o: o.__dict__))



from cqase.models import ResponseModel


class EnvironmentsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_ENV = "/environment/{}"
    _POST_CREATE_ENV = "/environment/{}"
    _GET_SPECIFIC_ENV = "/environment/{}/{}"
    _DELETE_ENV = "/environment/{}/{}"
    _UPDATE_ENV = "/environment/{}/{}"

    def get_all(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-environments
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_ENV.format(code)}",
        )

    def create(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-environment
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_ENV.format(code)}",
            json=body,
        )

    def get_specific_env(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-environment
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_SPECIFIC_ENV.format(code, uuid)}",
        )

    def delete(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-environment
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_ENV.format(code, uuid)}",
        )

    def update(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-environment
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_ENV.format(code, uuid)}",
            json=body,
        )



from cqase.models import ResponseModel


class ProjectsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_PROJECTS = "/project"
    _POST_CREATE_NEW_PROJECT = "/project"
    _GET_PROJECT_BY_CODE = "/project/{}"
    _DELETE_PROJECT_BY_CODE = "/project/{}"
    _POST_GRANT_ACCESS_PROJECT = "/project/{}/access"
    _DELETE_REVOKE_ACCESS_PROJECT = "/project/{}/access"

    def get_all(self) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-projects
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_PROJECTS}",
        )

    def create(self, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-project
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_NEW_PROJECT}",
            json=body,
        )

    def delete(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-project
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_PROJECT_BY_CODE.format(code)}",
        )

    def get_project_by_code(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-project
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_PROJECT_BY_CODE.format(code)}",
        )

    def grand_access(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/grant-access-to-project
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_GRANT_ACCESS_PROJECT.format(code)}",
            json=body,
        )

    def revoke_access(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/revoke-access-to-project
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}"
            f"{self._DELETE_REVOKE_ACCESS_PROJECT.format(code)}",
            json=body,
        )



import ntpath

from typing import Union, Tuple

from cqase.common.file_type import FileInfo
from cqase.models import ResponseModel


class AttachmentsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_ATTACHMENTS = "/attachment"
    _POST_UPLOAD_ATTACHMENTS = "/attachment/{}"
    _GET_ATTACHMENTS_BY_HASH = "/attachment/{}"
    _DELETE_ATTACHMENTS_BY_HASH = "/attachment/{}"

    def upload(
        self,
        code: str,
        *file_infos: Union[str, Tuple[str, str], Tuple[bytes, str, str]],
    ) -> ResponseModel:
        """
        https://developers.qase.io/reference/upload-attachment
        """
        files = []
        for _id, file in enumerate(file_infos):
            filename, path, mime, content = FileInfo.get_file_info(file)
            files.append((str(_id), (filename or ntpath.basename(path), content, mime)))

        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_UPLOAD_ATTACHMENTS.format(code)}",
            files=files,
        )

    def get_all(self) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-attachments
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_ATTACHMENTS}",
        )

    def remove_by_hash(self, hash: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-attachment
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_ATTACHMENTS_BY_HASH.format(hash)}",
        )

    def get_by_hash(self, hash: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-attachment
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ATTACHMENTS_BY_HASH.format(hash)}",
        )



from cqase.models import ResponseModel


class RunsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_RUNS = "/run/{}"
    _POST_CREATE_RUNS = "/run/{}"
    _GET_SPECIFIC_RUNS = "/run/{}/{}"
    _DELETE_RUNS = "/run/{}/{}"
    _UPDATE_RUNS = "/run/{}/{}/public"
    _POST_COMPLETE_RUNS = "/run/{}/{}/complete"

    def get_all(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-runs
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_RUNS.format(code)}",
        )

    def create(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-run
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_RUNS.format(code)}",
            json=body,
        )

    def get_run(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-run
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}" f"{self._GET_SPECIFIC_RUNS.format(code, uuid)}",
        )

    def delete(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-run
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_RUNS.format(code, uuid)}",
        )

    def update(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-run-publicity
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_RUNS.format(code, uuid)}",
            json=body,
        )

    def complete(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/complete-run
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_COMPLETE_RUNS.format(code, uuid)}",
        )



from cqase.models import ResponseModel


class CasesApi:
    def __init__(self, app):
        self.app = app

    _POST_CREATE_CASE = "/case/{}"
    _GET_ALL_TEST_CASES = "/case/{}"
    _GET_CASE_BY_ID = "/case/{}/{}"
    _DELETE_CASE = "/case/{}/{}"
    _UPDATE_CASE = "/case/{}/{}"

    def create(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-case
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_CASE.format(code)}",
            json=body,
        )

    def get_all(self, code: str, params: dict = None) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-cases
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_TEST_CASES.format(code)}",
            params=params,
        )

    def get_case_by_id(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-case
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_CASE_BY_ID.format(code, uuid)}",
        )

    def delete(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-case
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_CASE.format(code, uuid)}",
        )

    def update(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-case
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_CASE.format(code, uuid)}",
            json=body,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cqase', 'cqase.api', 'cqase.common']

package_data = \
{'': ['*']}

install_requires = \
['install>=1.3.5,<2.0.0', 'pip>=22.1.2,<23.0.0', 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'cqase-client',
    'version': '0.1.1',
    'description': 'Custom qase client',
    'long_description': '# custom-qase-client\n\n\nThis is an unofficial client [QASE](https://qase.io)\n\nOfficial client: https://github.com/qase-tms/qase-python\n\nAPI QASE: https://developers.qase.io/reference/\n\nGuide: https://developers.qase.io/docs\n\n### Installation\n\n------------\n\nYou can install via pip\n```\n$ pip install cqase-client\n```\nor with poetry\n```\n$ poetry add -D cqase-client\n```\n\n### How to work\n\n------------\n\nFirst, get api token from page https://app.qase.io/user/api/token (See guide)\n\nFor body and params use dict type, like in [requests](https://requests.readthedocs.io/en/latest/user/quickstart/#more-complicated-post-requests) library\n\n```python\nfrom cqase.client import QaseClient\n\nclient = QaseClient(api_token=\'YOUR_API_TOKEN\')\n\ncode = \'CODE\'  # projecet code\n# create project\nbody = {\'title\': f\'Title test project\', \'code\': code, \'access\': \'all\'}\nclient.projects.create(body=body)\n\n# create suite\nbody = {"title": "test suit"}\nclient.suites.create(code=code, body=body)\n\n# create case\nbody = {"title": "test case"}\nclient.cases.create(code=code, body=body)\n\n# create test run\nbody = {"title": "test run"}\nclient.runs.create(code=code, body=body)\n\n# upload attachment\nclient.attachments.upload(code, "./cat.jpeg")\n\n# finish test run\nclient.runs.complete(code=code, uuid=1)\n\n```\n\nIn work ...\n',
    'author': 'alexanderlozovoy',
    'author_email': 'berpress@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/berpress/custom-qase-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from cqase.models import ResponseModel


class ResultsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_RESULTS = "/result/{}"
    _POST_CREATE_RESULTS = "/result/{}/{}"
    _GET_SPECIFIC_RESULTS = "/result/{}/{}"
    _DELETE_RESULTS = "/result/{}/{}/{}"
    _UPDATE_RESULTS = "/result/{}/{}/{}"
    _POST_BULK_RESULTS = "/result/{}/{}/bulk"

    def get_all(self, code: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-results
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_RESULTS.format(code)}",
        )

    def create(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-result
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_RESULTS.format(code, uuid)}",
            json=body,
        )

    def get_result(self, code: str, hash_result: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-result
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}"
            f"{self._GET_SPECIFIC_RESULTS.format(code, hash_result)}",
        )

    def delete(self, code: str, uuid: int, hash_result: str) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-result
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}"
            f"{self._DELETE_RESULTS.format(code, uuid, hash_result)}",
        )

    def update(
        self, code: str, uuid: int, hash_result: str, body: dict
    ) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-result
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}"
            f"{self._UPDATE_RESULTS.format(code, uuid, hash_result)}",
            json=body,
        )

    def bulk(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-result-bulk
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_BULK_RESULTS.format(code, uuid)}",
            json=body,
        )



import mimetypes


class FileInfo:
    @staticmethod
    def get_file_info(file):
        filename = None
        if isinstance(file, tuple):
            if len(file) == 2:
                path, mime = file
            else:
                path, mime, filename = file
        else:
            path = file
            mime = mimetypes.guess_type(file)[0]
        if isinstance(path, bytes):
            content = path
        else:
            content = open(path, "rb")

        return filename, path, mime, content



from cqase.api.attachments import AttachmentsApi
from cqase.api.cases import CasesApi
from cqase.api.custom_fields import CustomFieldsApi
from cqase.api.defects import DefectsApi
from cqase.api.environments import EnvironmentsApi
from cqase.api.milestones import MilestoneApi
from cqase.api.plans import PlanesApi
from cqase.api.projects import ProjectsApi
from cqase.api.results import ResultsApi
from cqase.api.runs import RunsApi
from cqase.api.shared_steps import SharedStepsApi
from cqase.api.suites import SuitesApi
from cqase.request import Client


class QaseClient:
    def __init__(self, api_token: str, base_path: str = "https://api.qase.io/v1"):
        self.client = Client(headers={"Token": api_token})
        self.base_path = base_path

        self.projects = ProjectsApi(self)
        self.cases = CasesApi(self)
        self.attachments = AttachmentsApi(self)
        self.custom_fields = CustomFieldsApi(self)
        self.defects = DefectsApi(self)
        self.environments = EnvironmentsApi(self)
        self.milestones = MilestoneApi(self)
        self.planes = PlanesApi(self)
        self.runs = RunsApi(self)
        self.results = ResultsApi(self)
        self.shared_steps = SharedStepsApi(self)
        self.suites = SuitesApi(self)



from cqase.models import ResponseModel


class DefectsApi:
    def __init__(self, app):
        self.app = app

    _GET_ALL_DEFECTS = "/defect/{}"
    _POST_CREATE_NEW_DEFECTS = "/defect/{}"
    _GET_SPECIFIC_DEFECTS = "/defect/{}/{}"
    _DELETE_DEFECTS = "/defect/{}/{}"
    _UPDATE_DEFECTS = "/defect/{}/{}"
    _UPDATE_RESOLVE_DEFECT = "/defect/{}/resolve/{}"
    _UPDATE_DEFECT_STATUS = "/defect/{}/status/{}"

    def get_all(self, code: str):
        """
        https://developers.qase.io/reference/get-defects
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_ALL_DEFECTS.format(code)}",
        )

    def create(self, code: str, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/create-defect
        """
        return self.app.client.request(
            method="POST",
            url=f"{self.app.base_path}{self._POST_CREATE_NEW_DEFECTS.format(code)}",
            json=body,
        )

    def get_specific_defect(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/get-defect
        """
        return self.app.client.request(
            method="GET",
            url=f"{self.app.base_path}{self._GET_SPECIFIC_DEFECTS.format(code, uuid)}",
        )

    def delete(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/delete-defect
        """
        return self.app.client.request(
            method="DELETE",
            url=f"{self.app.base_path}{self._DELETE_DEFECTS.format(code, uuid)}",
        )

    def update(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-defect
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_DEFECTS.format(code, uuid)}",
            json=body,
        )

    def resolve_defect(self, code: str, uuid: int) -> ResponseModel:
        """
        https://developers.qase.io/reference/resolve-defect
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_RESOLVE_DEFECT.format(code, uuid)}",
        )

    def update_status(self, code: str, uuid: int, body: dict) -> ResponseModel:
        """
        https://developers.qase.io/reference/update-defect-status
        """
        return self.app.client.request(
            method="PATCH",
            url=f"{self.app.base_path}{self._UPDATE_DEFECT_STATUS.format(code, uuid)}",
            json=body,
        )
